@@ -718,7 +718,9 @@ _gnutls_raw_privkey_to_gkey (gnutls_privkey * key,
 	  gnutls_assert ();
 	  return GNUTLS_E_INIT_LIBEXTRA;
 	}
-      return _E_gnutls_openpgp_raw_privkey_to_gkey (key, raw_key);
+      return _E_gnutls_openpgp_raw_privkey_to_gkey (key, raw_key,
+						    (gnutls_openpgp_key_fmt_t)
+						    key_enc);
     default:
       gnutls_assert ();
       return GNUTLS_E_INTERNAL_ERROR;
